@@ -76,7 +76,9 @@ class Audalign:
         self.set_num_processors(num_processors)
         self.set_multiprocessing(multiprocessing)
         self.set_freq_threshold(freq_threshold)
+        self.hash_style = "panako_mod"
         self.set_hash_style(hash_style)
+        self.accuracy = 2
         self.set_accuracy(accuracy)
 
     def set_hash_style(self, hash_style: str) -> None:
@@ -105,6 +107,7 @@ class Audalign:
         Args
             accuracy (int): which accuracy level: 1-4
         """
+        accuracy = int(accuracy)
         if accuracy < 1 or accuracy > 4:
             print("Accuracy must be between 1 and 4")
             return
@@ -391,7 +394,7 @@ class Audalign:
         file_name, hashes = self._fingerprint_file(
             file_path, start_end=start_end, set_file_name=set_file_name, plot=plot
         )
-        if file_name != None:
+        if file_name is not None and hashes is not None:
             self.fingerprinted_files.append([file_name, hashes])
             self.file_names.append(file_name)
             self.total_fingerprints += len(hashes)
@@ -539,6 +542,7 @@ class Audalign:
         vert_scaling: float = 1.0,
         horiz_scaling: float = 1.0,
         calc_mse: bool = False,
+        max_lags: float = None,
         plot: bool = False,
     ) -> dict:
         """Recognize target file against against file visually.
@@ -579,6 +583,7 @@ class Audalign:
             vert_scaling=vert_scaling,
             horiz_scaling=horiz_scaling,
             calc_mse=calc_mse,
+            max_lags=max_lags,
             use_multiprocessing=self.multiprocessing,
             num_processes=self.num_processors,
             plot=plot,
@@ -641,7 +646,9 @@ class Audalign:
         vert_scaling: float = 1.0,
         horiz_scaling: float = 1.0,
         calc_mse: bool = False,
+        max_lags: float = None,
         plot: bool = False,
+        _file_audsegs: dict = None,
     ) -> dict:
         """Recognize target file against against directory visually.
         Uses image processing similarity techniques to identify areas with similar spectrums.
@@ -679,13 +686,15 @@ class Audalign:
             vert_scaling=vert_scaling,
             horiz_scaling=horiz_scaling,
             calc_mse=calc_mse,
+            max_lags=max_lags,
             use_multiprocessing=self.multiprocessing,
             num_processes=self.num_processors,
             plot=plot,
+            _file_audsegs=_file_audsegs,
         )
 
+    @staticmethod
     def write_processed_file(
-        self,
         file_path: str,
         destination_file: str,
         start_end: tuple = None,
@@ -749,7 +758,7 @@ class Audalign:
         start_end: tuple = None,
         write_extension: str = None,
         technique: str = "fingerprints",
-        alternate_strength_stat: str = None,
+        strength_stat: str = CONFIDENCE,
         filter_matches: float = None,
         locality: float = None,
         locality_filter_prop: float = None,
@@ -789,130 +798,29 @@ class Audalign:
         Returns:
             dict: dict of file name with shift as value along with match info
         """
-        self.file_names, temp_file_names = [], self.file_names
-        self.fingerprinted_files, temp_fingerprinted_files = (
-            [],
-            self.fingerprinted_files,
+
+        return align._align(
+            self,
+            filename_list=[target_file],
+            file_dir=directory_path,
+            destination_path=destination_path,
+            target_aligning=True,
+            target_start_end=start_end,
+            write_extension=write_extension,
+            technique=technique,
+            alternate_strength_stat=strength_stat,
+            filter_matches=filter_matches,
+            locality=locality,
+            locality_filter_prop=locality_filter_prop,
+            volume_threshold=volume_threshold,
+            volume_floor=volume_floor,
+            vert_scaling=vert_scaling,
+            horiz_scaling=horiz_scaling,
+            img_width=img_width,
+            calc_mse=calc_mse,
+            cor_sample_rate=cor_sample_rate,
+            **kwargs,
         )
-        self.total_fingerprints, temp_total_fingerprints = 0, self.total_fingerprints
-
-        try:
-            if alternate_strength_stat == "mse":
-                calc_mse = True
-
-            target_name = os.path.basename(target_file)
-            total_alignment = {}
-            file_names_and_paths = {}
-
-            if technique == "fingerprints":
-
-                if filter_matches is None:
-                    filter_matches = 1
-                if locality_filter_prop is None or locality_filter_prop > 1.0:
-                    locality_filter_prop = 1.0
-
-                if start_end is not None:
-                    self.fingerprint_file(target_file, start_end=start_end)
-
-                all_against_files = filehandler.find_files(directory_path)
-                all_against_files_full = [x[0] for x in all_against_files]
-                all_against_files_base = [
-                    os.path.basename(x) for x in all_against_files_full
-                ]
-                if (
-                    os.path.basename(target_file) in all_against_files_base
-                    and target_file not in all_against_files_full
-                ):
-                    self.fingerprint_file(target_file)
-                self.fingerprint_directory(directory_path)
-
-                alignment = self.recognize(
-                    target_file,
-                    filter_matches=filter_matches,
-                    locality=locality,
-                    locality_filter_prop=locality_filter_prop,
-                    start_end=start_end,
-                )
-
-            elif technique == "visual":
-                alignment = self.visrecognize_directory(
-                    target_file_path=target_file,
-                    against_directory=directory_path,
-                    start_end=start_end,
-                    volume_threshold=volume_threshold,
-                    volume_floor=volume_floor,
-                    vert_scaling=vert_scaling,
-                    horiz_scaling=horiz_scaling,
-                    img_width=img_width,
-                    calc_mse=calc_mse,
-                )
-            elif technique == "correlation":
-                alignment = self.correcognize_directory(
-                    target_file_path=target_file,
-                    against_directory=directory_path,
-                    start_end=start_end,
-                    filter_matches=filter_matches,
-                    sample_rate=cor_sample_rate,
-                    **kwargs,
-                )
-            else:
-                raise NameError(
-                    f'Technique parameter must be fingerprint, visual, or correlation, not "{technique}"'
-                )
-
-            file_names_and_paths[target_name] = target_file
-            total_alignment[target_name] = alignment
-
-            if not alignment:
-                print("No results")
-                return
-
-            for file_path, _ in filehandler.find_files(directory_path):
-                if (
-                    os.path.basename(file_path)
-                    in total_alignment[target_name]["match_info"].keys()
-                ):
-                    file_names_and_paths[os.path.basename(file_path)] = file_path
-
-            if not alternate_strength_stat:
-                if technique == "fingerprints":
-                    alternate_strength_stat = self.CONFIDENCE
-                elif technique == "visual":
-                    alternate_strength_stat = "ssim"
-                else:
-                    alternate_strength_stat = self.CONFIDENCE
-            files_shifts = align.find_most_matches(
-                total_alignment, strength_stat=alternate_strength_stat
-            )
-            if not files_shifts:
-                return
-
-            if destination_path:
-                try:
-                    # Make target directory
-                    if not os.path.exists(destination_path):
-                        os.makedirs(destination_path)
-                    self._write_shifted_files(
-                        files_shifts,
-                        destination_path,
-                        file_names_and_paths,
-                        write_extension,
-                    )
-                except PermissionError:
-                    print("Permission Denied for write align")
-
-            print(
-                f"{len(files_shifts)} out of {len(file_names_and_paths)} found and aligned"
-            )
-
-            files_shifts["match_info"] = total_alignment
-            files_shifts["names_and_paths"] = file_names_and_paths
-            return files_shifts
-
-        finally:
-            self.file_names = temp_file_names
-            self.fingerprinted_files = temp_fingerprinted_files
-            self.total_fingerprints = temp_total_fingerprints
 
     def align_files(
         self,
@@ -927,6 +835,13 @@ class Audalign:
         locality_filter_prop: float = None,
         cor_sample_rate: int = fingerprint.DEFAULT_FS,
         max_lags: float = None,
+        strength_stat: str = CONFIDENCE,
+        volume_threshold: float = 216,
+        volume_floor: float = 10.0,
+        vert_scaling: float = 1.0,
+        horiz_scaling: float = 1.0,
+        img_width: float = 1.0,
+        calc_mse: bool = False,
         **kwargs,
     ):
         """
@@ -953,9 +868,8 @@ class Audalign:
             files_shifts (dict{float}): dict of file name with shift as value
         """
         filename_list = [filename_a, filename_b, *filenames]
-        # print(filename_a, filename_b, *filenames, sep="\n")
-        # print(filename_list)
-        return self._align(
+        return align._align(
+            ada_obj=self,
             filename_list=filename_list,
             file_dir=None,
             destination_path=destination_path,
@@ -965,6 +879,13 @@ class Audalign:
             locality=locality,
             locality_filter_prop=locality_filter_prop,
             cor_sample_rate=cor_sample_rate,
+            alternate_strength_stat=strength_stat,
+            volume_threshold=volume_threshold,
+            volume_floor=volume_floor,
+            vert_scaling=vert_scaling,
+            horiz_scaling=horiz_scaling,
+            img_width=img_width,
+            calc_mse=calc_mse,
             max_lags=max_lags,
             **kwargs,
         )
@@ -980,6 +901,13 @@ class Audalign:
         locality_filter_prop: float = None,
         cor_sample_rate: int = fingerprint.DEFAULT_FS,
         max_lags: float = None,
+        strength_stat: str = CONFIDENCE,
+        volume_threshold: float = 216,
+        volume_floor: float = 10.0,
+        vert_scaling: float = 1.0,
+        horiz_scaling: float = 1.0,
+        img_width: float = 1.0,
+        calc_mse: bool = False,
         **kwargs,
     ):
         """
@@ -1000,7 +928,8 @@ class Audalign:
         -------
             files_shifts (dict{float}): dict of file name with shift as value
         """
-        return self._align(
+        return align._align(
+            ada_obj=self,
             filename_list=None,
             file_dir=directory_path,
             destination_path=destination_path,
@@ -1010,137 +939,17 @@ class Audalign:
             locality=locality,
             locality_filter_prop=locality_filter_prop,
             cor_sample_rate=cor_sample_rate,
+            alternate_strength_stat=strength_stat,
+            volume_threshold=volume_threshold,
+            volume_floor=volume_floor,
+            vert_scaling=vert_scaling,
+            horiz_scaling=horiz_scaling,
+            img_width=img_width,
+            calc_mse=calc_mse,
             max_lags=max_lags,
             **kwargs,
         )
 
-    def _align(
-        self,
-        filename_list: str,
-        file_dir: str,
-        destination_path: str = None,
-        write_extension: str = None,
-        technique: str = "fingerprints",
-        filter_matches: float = None,
-        locality: float = None,
-        locality_filter_prop: float = None,
-        cor_sample_rate: int = fingerprint.DEFAULT_FS,
-        max_lags: float = None,
-        fine_aud_file_dict: dict = None,
-        **kwargs,
-    ):
-
-        self.file_names, temp_file_names = [], self.file_names
-        self.fingerprinted_files, temp_fingerprinted_files = (
-            [],
-            self.fingerprinted_files,
-        )
-        self.total_fingerprints, temp_total_fingerprints = 0, self.total_fingerprints
-
-        try:
-
-            # Make target directory
-            if destination_path:
-                if not os.path.exists(destination_path):
-                    os.makedirs(destination_path)
-
-            if technique == "fingerprints":
-                if filter_matches is None:
-                    filter_matches = 1
-                if locality_filter_prop is None or locality_filter_prop > 1.0:
-                    locality_filter_prop = 1.0
-                if file_dir:
-                    self.fingerprint_directory(file_dir)
-                else:
-                    self.fingerprint_directory(
-                        filename_list,
-                        _file_audsegs=fine_aud_file_dict,
-                    )
-            elif technique == "correlation":
-                if file_dir:
-                    self.file_names = filehandler.get_audio_files_directory(file_dir)
-                elif fine_aud_file_dict:
-                    self.file_names = [
-                        os.path.basename(x) for x in fine_aud_file_dict.keys()
-                    ]
-                else:
-                    self.file_names = [os.path.basename(x) for x in filename_list]
-
-            else:
-                raise NameError(
-                    f'Technique parameter must be fingerprints, visual, or correlation, not "{technique}"'
-                )
-
-            total_alignment = {}
-            file_names_and_paths = {}
-
-            if file_dir:
-                file_list = filehandler.find_files(file_dir)
-                dir_or_list = file_dir
-            elif fine_aud_file_dict:
-                file_list = zip(
-                    fine_aud_file_dict.keys(), ["_"] * len(fine_aud_file_dict)
-                )
-                dir_or_list = fine_aud_file_dict.keys()
-            else:
-                file_list = zip(filename_list, ["_"] * len(filename_list))
-                dir_or_list = filename_list
-            # Get matches and paths
-            for file_path, _ in file_list:
-                name = os.path.basename(file_path)
-                if name in self.file_names:
-                    if technique == "fingerprints":
-                        alignment = self.recognize(
-                            file_path,
-                            filter_matches=filter_matches,
-                            locality=locality,
-                            locality_filter_prop=locality_filter_prop,
-                            max_lags=max_lags,
-                        )
-                    elif technique == "correlation":
-                        alignment = self.correcognize_directory(
-                            file_path,
-                            dir_or_list,
-                            filter_matches=filter_matches,
-                            sample_rate=cor_sample_rate,
-                            _file_audsegs=fine_aud_file_dict,
-                            max_lags=max_lags,
-                            **kwargs,
-                        )
-                    file_names_and_paths[name] = file_path
-                    total_alignment[name] = alignment
-
-            files_shifts = align.find_most_matches(total_alignment)
-            if not files_shifts:
-                return
-            files_shifts = align.find_matches_not_in_file_shifts(
-                total_alignment, files_shifts
-            )
-
-            if destination_path:
-                try:
-                    self._write_shifted_files(
-                        files_shifts,
-                        destination_path,
-                        file_names_and_paths,
-                        write_extension,
-                    )
-                except PermissionError:
-                    print("Permission Denied for write align")
-
-            print(
-                f"{len(files_shifts)} out of {len(file_names_and_paths)} found and aligned"
-            )
-
-            files_shifts["match_info"] = total_alignment
-            files_shifts["names_and_paths"] = file_names_and_paths
-            return files_shifts
-
-        finally:
-            self.file_names = temp_file_names
-            self.fingerprinted_files = temp_fingerprinted_files
-            self.total_fingerprints = temp_total_fingerprints
-
     def fine_align(
         self,
         results,
@@ -1154,6 +963,12 @@ class Audalign:
         cor_sample_rate: int = fingerprint.DEFAULT_FS,
         match_index: int = 0,
         strength_stat: str = CONFIDENCE,
+        volume_threshold: float = 216,
+        volume_floor: float = 10.0,
+        vert_scaling: float = 1.0,
+        horiz_scaling: float = 1.0,
+        img_width: float = 1.0,
+        calc_mse: bool = False,
         **kwargs,
     ):
         """
@@ -1171,6 +986,7 @@ class Audalign:
             cor_sample_rate (int): Sampling rate for correlation
             match_index (int): reorders the input results to the given match index.
             strength_stat (str): strength stat for finding proper alignments. Defaults to confidence.
+            # TODO Docs
             **kwargs: Additional arguments for finding peaks in correlation
 
         Returns
@@ -1179,6 +995,14 @@ class Audalign:
         """
         print("Fine Aligning...")
 
+        if strength_stat == "confidence":
+            _ = results["match_info"]
+            _ = _[list(_.keys())[0]]
+            _ = _["match_info"]
+            _ = _[list(_.keys())[0]]
+            if "confidence" not in _.keys():
+                strength_stat = "ssim"
+
         if match_index != 0:
             recalc_shifts_results = align.recalc_shifts_index(
                 results, strength_stat=strength_stat, match_index=match_index
@@ -1191,7 +1015,8 @@ class Audalign:
                 results, sample_rate=cor_sample_rate
             )
 
-        new_results = self._align(
+        new_results = align._align(
+            ada_obj=self,
             filename_list=None,
             file_dir=None,
             technique=technique,
@@ -1201,6 +1026,13 @@ class Audalign:
             cor_sample_rate=cor_sample_rate,
             max_lags=max_lags,
             fine_aud_file_dict=paths_audio,
+            alternate_strength_stat=strength_stat,
+            volume_threshold=volume_threshold,
+            volume_floor=volume_floor,
+            vert_scaling=vert_scaling,
+            horiz_scaling=horiz_scaling,
+            img_width=img_width,
+            calc_mse=calc_mse,
             **kwargs,
         )
         if new_results is None:
