@@ -1,9 +1,10 @@
 import audalign.fingerprint as fingerprint
-from audalign.filehandler import read, find_files
+from audalign.filehandler import read, find_files, get_shifted_file
 from pydub.exceptions import CouldntDecodeError
 import tqdm
 import time
 import os
+import sys
 from skimage.metrics import structural_similarity as ssim
 from skimage.metrics import mean_squared_error
 import matplotlib.pyplot as plt
@@ -29,6 +30,7 @@ def visrecognize(
     vert_scaling: float = 1.0,
     horiz_scaling: float = 1.0,
     calc_mse=False,
+    max_lags: float = None,
     use_multiprocessing=True,
     num_processes=None,
     plot=False,
@@ -77,6 +79,7 @@ def visrecognize(
         calc_mse=calc_mse,
         use_multiprocessing=use_multiprocessing,
         num_processes=num_processes,
+        max_lags=max_lags,
     )
     file_match = process_results(
         results_list,
@@ -106,6 +109,86 @@ def visrecognize(
 # ------------------------------------------------------------------------------------------
 
 
+def _visrecognize_directory(
+    file_path: str,
+    target_file_path: str,
+    start_end: tuple = None,
+    img_width=1.0,
+    volume_threshold=215.0,
+    volume_floor: float = 10.0,
+    vert_scaling: float = 1.0,
+    horiz_scaling: float = 1.0,
+    calc_mse: bool = False,
+    max_lags: float = None,
+    use_multiprocessing: bool = True,
+    num_processes: int = None,
+    plot: bool = False,
+    _file_audsegs: dict = None,
+):
+    if type(target_file_path) == str:
+        target_arr2d, transposed_target_arr2d = get_arrays(
+            target_file_path,
+            volume_floor=volume_floor,
+            vert_scaling=vert_scaling,
+            horiz_scaling=horiz_scaling,
+            start_end=start_end,
+            _file_audsegs=_file_audsegs,
+        )
+
+        target_index_list = find_index_arr(
+            transposed_target_arr2d, volume_threshold, img_width
+        )
+    else:
+        (
+            target_file_path,
+            target_arr2d,
+            transposed_target_arr2d,
+            target_index_list,
+        ) = target_file_path
+
+    file_path, _ = file_path
+
+    if os.path.basename(file_path) == os.path.basename(target_file_path):
+        return {}
+    try:
+        against_arr2d, transposed_against_arr2d = get_arrays(
+            file_path,
+            volume_floor=volume_floor,
+            vert_scaling=vert_scaling,
+            horiz_scaling=horiz_scaling,
+            _file_audsegs=_file_audsegs,
+        )
+        results_list = _visrecognize(
+            transposed_target_arr2d=transposed_target_arr2d,
+            target_file_path=target_file_path,
+            target_index_list=target_index_list,
+            against_file_path=file_path,
+            transposed_against_arr2d=transposed_against_arr2d,
+            img_width=img_width,
+            volume_threshold=volume_threshold,
+            calc_mse=calc_mse,
+            use_multiprocessing=use_multiprocessing,
+            num_processes=num_processes,
+            max_lags=max_lags,
+        )
+        single_file_match = process_results(
+            results_list,
+            os.path.basename(file_path),
+            horiz_scaling=horiz_scaling,
+        )
+        if plot:
+            plot_two_images(
+                target_arr2d,
+                against_arr2d,
+                imgA_title=os.path.basename(target_file_path),
+                imgB_title=os.path.basename(file_path),
+            )
+        return single_file_match
+    except CouldntDecodeError:
+        print(f'File "{file_path}" could not be decoded')
+        return {}
+
+
 def visrecognize_directory(
     target_file_path: str,
     against_directory: str,
@@ -115,10 +198,13 @@ def visrecognize_directory(
     volume_floor: float = 10.0,
     vert_scaling: float = 1.0,
     horiz_scaling: float = 1.0,
-    calc_mse=False,
-    use_multiprocessing=True,
-    num_processes=None,
-    plot=False,
+    calc_mse: bool = False,
+    max_lags: float = None,
+    use_multiprocessing: bool = True,
+    num_processes: int = None,
+    plot: bool = False,
+    _file_audsegs: dict = None,
+    _include_filename=False,
 ):
     # With frequency of 44100
     # Each frame is 0.0929 seconds with an overlap ratio of .5,
@@ -134,60 +220,71 @@ def visrecognize_directory(
 
     img_width = get_frame_width(img_width)
 
-    target_arr2d, transposed_target_arr2d = get_arrays(
-        target_file_path,
+    if use_multiprocessing == False:
+        target_arr2d, transposed_target_arr2d = get_arrays(
+            target_file_path,
+            volume_floor=volume_floor,
+            vert_scaling=vert_scaling,
+            horiz_scaling=horiz_scaling,
+            start_end=start_end,
+            _file_audsegs=_file_audsegs,
+        )
+
+        target_index_list = find_index_arr(
+            transposed_target_arr2d, volume_threshold, img_width
+        )
+
+        target_file_path = (
+            target_file_path,
+            target_arr2d,
+            transposed_target_arr2d,
+            target_index_list,
+        )
+
+    if type(against_directory) == str:
+        against_files = list(find_files(against_directory))
+    else:
+        against_files = list(zip(against_directory, ["_"] * len(against_directory)))
+
+    _visrecognize_directory_ = partial(
+        _visrecognize_directory,
+        target_file_path=target_file_path,
+        start_end=start_end,
+        img_width=img_width,
+        volume_threshold=volume_threshold,
         volume_floor=volume_floor,
         vert_scaling=vert_scaling,
         horiz_scaling=horiz_scaling,
-        start_end=start_end,
+        calc_mse=calc_mse,
+        max_lags=max_lags,
+        use_multiprocessing=False,
+        num_processes=num_processes,
+        plot=plot,
+        _file_audsegs=_file_audsegs,
     )
 
-    target_index_list = find_index_arr(
-        transposed_target_arr2d, volume_threshold, img_width
-    )
-
-    against_files = find_files(against_directory)
-    file_match = {}
-
-    for file_path, _ in against_files:
-
-        if os.path.basename(file_path) == os.path.basename(target_file_path):
-            continue
+    if use_multiprocessing == False:
+        results_list = []
+        for file_path in against_files:
+            results_list += [_visrecognize_directory_(file_path)]
+    else:
         try:
-            against_arr2d, transposed_against_arr2d = get_arrays(
-                file_path,
-                volume_floor=volume_floor,
-                vert_scaling=vert_scaling,
-                horiz_scaling=horiz_scaling,
+            nprocesses = num_processes or multiprocessing.cpu_count()
+        except NotImplementedError:
+            nprocesses = 1
+        else:
+            nprocesses = 1 if nprocesses <= 0 else nprocesses
+
+        with multiprocessing.Pool(nprocesses) as pool:
+            results_list = pool.map(
+                _visrecognize_directory_, tqdm.tqdm(list(against_files))
             )
-            results_list = _visrecognize(
-                transposed_target_arr2d=transposed_target_arr2d,
-                target_file_path=target_file_path,
-                target_index_list=target_index_list,
-                against_file_path=file_path,
-                transposed_against_arr2d=transposed_against_arr2d,
-                img_width=img_width,
-                volume_threshold=volume_threshold,
-                calc_mse=calc_mse,
-                use_multiprocessing=use_multiprocessing,
-                num_processes=num_processes,
-            )
-            single_file_match = process_results(
-                results_list,
-                os.path.basename(file_path),
-                horiz_scaling=horiz_scaling,
-            )
-            if plot:
-                plot_two_images(
-                    target_arr2d,
-                    against_arr2d,
-                    imgA_title=os.path.basename(target_file_path),
-                    imgB_title=os.path.basename(file_path),
-                )
-            if single_file_match:
-                file_match = {**file_match, **single_file_match}
-        except CouldntDecodeError:
-            print(f'File "{file_path}" could not be decoded')
+            pool.close()
+            pool.join()
+
+    file_match = {}
+    for i in results_list:
+        file_match = {**file_match, **i}
 
     t = time.time() - t
 
@@ -195,6 +292,11 @@ def visrecognize_directory(
     if file_match:
         result["match_time"] = t
         result["match_info"] = file_match
+        if _include_filename:
+            if type(target_file_path) == tuple:
+                result["filename"] = target_file_path[0]
+            else:
+                result["filename"] = target_file_path
         return result
 
     return None
@@ -214,10 +316,11 @@ def _visrecognize(
     calc_mse=False,
     use_multiprocessing=True,
     num_processes=None,
+    max_lags: float = None,
 ):
 
-    th, _ = transposed_target_arr2d.shape
-    ah, _ = transposed_against_arr2d.shape
+    # th, _ = transposed_target_arr2d.shape
+    # ah, _ = transposed_against_arr2d.shape
 
     # plot_two_images(transposed_target_arr2d, transposed_against_arr2d)
 
@@ -233,15 +336,14 @@ def _visrecognize(
         transposed_against_arr2d, volume_threshold, img_width
     )
 
-    index_list = pair_index_tuples(target_index_list, against_index_list)
+    index_list = pair_index_tuples(
+        target_index_list, against_index_list, max_lags=max_lags
+    )
 
     # offsets = [x[0] - x[1] for x in index_list]
     # print()
     # print(offsets.count(215))
 
-    # print()
-    # print(len(index_list))
-
     _calculate_comp_values = partial(
         calculate_comp_values,
         img_width=img_width,
@@ -249,7 +351,7 @@ def _visrecognize(
     )
 
     # calculate all mse and ssim values
-    if use_multiprocessing == True:
+    if use_multiprocessing == True and sys.platform != "linux":
 
         try:
             nprocesses = num_processes or multiprocessing.cpu_count()
@@ -258,18 +360,11 @@ def _visrecognize(
         else:
             nprocesses = 1 if nprocesses <= 0 else nprocesses
 
-        print(nprocesses)  # FIXME
-
         index_list = divy_index_list(
             index_list, transposed_target_arr2d, transposed_against_arr2d, nprocesses
         )
-        # index_list = index_list[:10]
 
-        # multiprocessing.set_start_method("fork")
-
-        # with multiprocessing.get_context("fork").Pool(nprocesses) as pool:
         with multiprocessing.Pool(nprocesses) as pool:
-            # pool = multiprocessing.get_context("spawn").Pool(nprocesses)
             results_list = pool.map(_calculate_comp_values, tqdm.tqdm(list(index_list)))
             pool.close()
             pool.join()
@@ -278,7 +373,7 @@ def _visrecognize(
             index_list, transposed_target_arr2d, transposed_against_arr2d, 1
         )
         results_list = []
-        for i in tqdm.tqdm(list(index_list)):
+        for i in list(index_list):
             results_list += [_calculate_comp_values(i)]
     new_results_list = []
     for i in results_list:
@@ -291,7 +386,6 @@ def _visrecognize(
 
 
 def divy_index_list(index_list, target_arr, against_arr, nprocesses):
-    index_list = index_list[:6]
     sublists = [list(index_list)[i::nprocesses] for i in range(nprocesses)]
     return zip(sublists, [target_arr] * nprocesses, [against_arr] * nprocesses)
 
@@ -302,8 +396,12 @@ def get_arrays(
     vert_scaling: float = 1.0,
     horiz_scaling: float = 1.0,
     start_end: tuple = None,
+    _file_audsegs: dict = None,
 ):
-    samples, _ = read(file_path, start_end=start_end)
+    if _file_audsegs is not None:
+        samples = get_shifted_file(file_path, _file_audsegs[file_path])
+    else:
+        samples, _ = read(file_path, start_end=start_end)
     arr2d = fingerprint.fingerprint(samples, retspec=True)
     if fingerprint.threshold > 0:
         arr2d = arr2d[0 : -fingerprint.threshold]
@@ -353,11 +451,28 @@ def find_index_arr(arr2d, threshold, img_width):
     return index_list
 
 
-def pair_index_tuples(target_list, against_list):
+def pair_index_tuples(target_list, against_list, max_lags: float = None):
     index_pairs = []
-    for i in target_list:
-        for j in against_list:
-            index_pairs += [(i, j)]
+    if max_lags is None:
+        for i in target_list:
+            for j in against_list:
+                index_pairs += [(i, j)]
+    else:
+        max_lags = max(  # turns into frames
+            int(
+                max_lags
+                // (
+                    fingerprint.DEFAULT_WINDOW_SIZE
+                    / fingerprint.DEFAULT_FS
+                    * fingerprint.DEFAULT_OVERLAP_RATIO
+                )
+            ),
+            1,
+        )
+        for i in target_list:
+            for j in against_list:
+                if abs(j - i) <= max_lags:
+                    index_pairs += [(i, j)]
     return index_pairs
 
 
@@ -445,7 +560,9 @@ def process_results(results_list, filename, horiz_scaling: float = 1.0):
     match_offsets = sorted(
         match_offsets,
         reverse=True,
-        key=lambda x: (np.log2(x[0][2] + 1) * (np.log(x[0][1] + 1) / np.log(1.5))),
+        key=lambda x: (x[0][1], x[0][2]),
+        # key=lambda x: (x[0][2], x[0][1]),
+        # key=lambda x: (np.log2(x[0][2] + 1) * (np.log(x[0][1] + 100) / np.log(2.5))),
     )  # sort by ssim must be reversed for ssim
     # match_offsets, reverse=True, key=lambda x: (x[0][2], x[0][1])
     # match_offsets, reverse=True, key=lambda x: x[0][2] sorts by num matches
